# opsinpy

Python bindings for OPSIN (Open Parser for Systematic IUPAC Nomenclature) using JPype for direct Java integration.

## Overview

This package provides Python bindings for OPSIN, allowing you to convert IUPAC chemical names to various molecular representations (SMILES, CML, InChI, etc.) directly from Python without subprocess overhead.

## Features

- Direct Java integration using JPype (no subprocess overhead)
- Support for all OPSIN output formats: SMILES, CML, InChI, StdInChI, StdInChIKey
- Batch processing support
- Full access to OPSIN configuration options
- Type hints and comprehensive error handling
- Compatible with Python 3.8+

## Installation

### From PyPI (Recommended)

```bash
pip install opsinpy
```

### From Source

```bash
git clone https://github.com/craabreu/opsinpy.git
cd opsinpy
pip install -e .
```

**Requirements:**
- Python 3.8+
- Java 8+ (required by OPSIN)
- JPype1 (automatically installed)

## Quick Start

```python
from opsinpy import OpsinPy

# Initialize the converter
opsin = OpsinPy()

# Convert a single chemical name
smiles = opsin.name_to_smiles("ethane")
print(smiles)  # "CC"

# Convert to different formats
cml = opsin.name_to_cml("acetamide")
inchi = opsin.name_to_inchi("benzene")

# Batch conversion
names = ["methane", "ethane", "propane"]
smiles_list = opsin.names_to_smiles(names)
print(smiles_list)  # ["C", "CC", "CCC"]

# With custom options
smiles = opsin.name_to_smiles("ethyl", allow_radicals=True)
```

## Performance and Benchmarks

This implementation provides **dramatic performance improvements** over subprocess-based approaches like `py2opsin`:

### Key Performance Benefits

- **2-1000x faster** than py2opsin depending on workload
- **One-time JVM startup** vs subprocess overhead per call
- **Excellent batch processing** performance
- **100% compatible** results with existing tools

### Benchmark Results Summary

| Processing Mode | opsinpy | py2opsin | opsinpy Advantage |
|-----------------|----------|----------|--------------------|
| Individual compounds | ~1-3ms each | ~500ms each | **10-900x faster** |
| Batch processing | ~2-4ms total | ~500ms total | **130-250x faster** |
| py2opsin improvement | - | Individual→Batch: **5-9x faster** | - |

### Real-World Throughput (39 molecules including complex IUPAC)

| Metric | opsinpy | py2opsin | Advantage |
|--------|----------|----------|-----------|
| **Throughput** | 2,835 molecules/sec | 73 molecules/sec | **38.7x higher** |
| **Time per molecule** | 0.35ms | 13.65ms | **39x faster** |

**Key Finding**: Both libraries benefit from batch processing, but opsinpy delivers superior performance in all scenarios.

### Running the Benchmark

Compare performance on your system:

```bash
# Automatic setup and benchmark
python benchmarks/run_benchmark.py

# Or manually install py2opsin and run
pip install py2opsin
python benchmarks/benchmark_comparison.py
```

The benchmark uses publicly available packages from PyPI for complete transparency and reproducibility. See `benchmarks/BENCHMARK_RESULTS.md` for detailed analysis.

## API Reference

### OpsinPy Class

The main class for chemical name conversion.

#### Methods

- `name_to_smiles(name, **options)` - Convert to SMILES
- `name_to_cml(name, **options)` - Convert to CML
- `name_to_inchi(name, **options)` - Convert to InChI
- `name_to_stdinchi(name, **options)` - Convert to StdInChI
- `name_to_stdinchikey(name, **options)` - Convert to StdInChIKey
- `names_to_smiles(names, **options)` - Batch convert to SMILES
- `names_to_cml(names, **options)` - Batch convert to CML
- `names_to_inchi(names, **options)` - Batch convert to InChI
- `names_to_stdinchi(names, **options)` - Batch convert to StdInChI
- `names_to_stdinchikey(names, **options)` - Batch convert to StdInChIKey

#### Options

All conversion methods accept the following optional parameters:

- `allow_acids_without_acid` (bool): Allow interpretation of acids without the word "acid"
- `allow_radicals` (bool): Enable radical interpretation
- `allow_uninterpretable_stereo` (bool): Allow ignoring uninterpretable stereochemistry
- `detailed_failure_analysis` (bool): Provide detailed failure analysis

## Development

### Running Tests

```bash
# Install development dependencies
pip install -e ".[dev]"

# Run all tests
pytest

# Run only unit tests
pytest -m unit

# Run only integration tests
pytest -m integration

# Run tests excluding slow tests
pytest -m "not slow"
```

### Test Structure

The test suite includes:
- **Unit tests**: Fast tests for individual components
- **Integration tests**: Tests that involve OPSIN Java integration
- **Edge case tests**: Tests for error conditions and boundary cases
- **Exception tests**: Tests for custom exception classes

### Code Quality

This project uses:
- **Ruff** for linting and import sorting
- **Black** for code formatting
- **MyPy** for type checking
- **Pytest** for testing

## Authors

- **Charlles Abreu** <craabreu@mit.edu> - Project conception and direction
- **Claude 3.5 Sonnet (Anthropic AI)** - Initial implementation and documentation
- **Claude 4 Sonnet (Anthropic AI)** - Code optimization and modern packaging

## License

This package is distributed under the MIT License, same as OPSIN.

## Citation

If you use this package in your research, please cite the original OPSIN paper:

```
Chemical Name to Structure: OPSIN, an Open Source Solution
Daniel M. Lowe, Peter T. Corbett, Peter Murray-Rust, Robert C. Glen
Journal of Chemical Information and Modeling 2011 51 (3), 739-753
https://doi.org/10.1021/ci100384d
```

## Acknowledgments

This project builds upon the excellent work of:
- The [OPSIN team](https://opsin.ch.cam.ac.uk/) at the University of Cambridge
- The [JPype project](https://github.com/jpype-project/jpype) for Java-Python integration

## Contributing

Contributions are welcome! Please feel free to submit issues and pull requests.

## Links

- [GitHub Repository](https://github.com/craabreu/opsinpy)
- [OPSIN Source Code](https://github.com/dan2097/opsin)
- [OPSIN Web Interface](https://opsin.ch.cam.ac.uk/)
- [OPSIN Documentation](https://opsin.ch.cam.ac.uk/) 